@@ -489,6 +489,7 @@ build_srpm(){
     sed -i "/^%changelog/a * $(date "+%a") $(date "+%b") $(date "+%d") $(date "+%Y") Percona Development Team <info@percona.com> - ${VERSION}-${RELEASE}" percona-server.spec
     #
     cd ${WORKDIR}/rpmbuild/SOURCES
+    wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
     #wget http://downloads.sourceforge.net/boost/${BOOST_PACKAGE_NAME}.tar.bz2
     wget --no-check-certificate http://jenkins.percona.com/downloads/boost/${BOOST_PACKAGE_NAME}.tar.gz
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/*.patch' --strip=3
@@ -499,6 +500,20 @@ build_srpm(){
     #
     cd ${WORKDIR}
     #
+
+    cd ${WORKDIR}/rpmbuild/SPECS
+        line_number=$(grep -n SOURCE999 percona-server.spec | awk -F ':' '{print $1}')
+        cp ../SOURCES/call-home.sh ./
+        awk -v n=$line_number 'NR <= n {print > "part1.txt"} NR > n {print > "part2.txt"}' percona-server.spec
+        head -n -1 part1.txt > temp && mv temp part1.txt
+        echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> part1.txt
+        cat call-home.sh >> part1.txt 
+        echo "CALLHOME" >> part1.txt
+        cat part2.txt >> part1.txt
+        rm -f call-home.sh part2.txt
+        mv part1.txt percona-server.spec
+    cd ${WORKDIR}
+
     mv -fv ${TARFILE} ${WORKDIR}/rpmbuild/SOURCES
     #
     rpmbuild -bs --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .generic" rpmbuild/SPECS/percona-server.spec
@@ -732,6 +747,19 @@ build_deb(){
     fi
     dch -b -m -D "$DEBIAN_VERSION" --force-distribution -v "${VERSION}-${RELEASE}-${DEB_RELEASE}.${DEBIAN_VERSION}" 'Update distribution'
 
+    cd debian/
+        wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
+        sed -i 's:exit 0::' percona-server-server-5.7.postinst
+        echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> percona-server-server-5.7.postinst
+        cat call-home.sh >> percona-server-server-5.7.postinst 
+        echo "CALLHOME" >> percona-server-server-5.7.postinst
+        echo 'bash -x /tmp/call-home.sh -f "PRODUCT_FAMILY_PS" -v "5.7.44-48-1" -d "PACKAGE" &>/dev/null || :' >> percona-server-server-5.7.postinst
+        echo "rm -rf /tmp/call-home.sh" >> percona-server-server-5.7.postinst
+        echo "exit 0" >> percona-server-server-5.7.postinst
+        rm -f call-home.sh
+    cd ../
+
+
     if [ ${ARCH} != "x86_64" ]; then
         rm -f debian/rules
         rm -f debian/control
