@@ -1,4 +1,4 @@
-/* Copyright (c) 2018 Percona LLC and/or its affiliates. All rights reserved.
+/* Copyright (c) 2018, 2021 Percona LLC and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -16,19 +16,16 @@
 
 #include "vault_credentials.h"
 
-namespace keyring
-{
+namespace keyring {
 
-static Secure_string empty_value;
-
-const Secure_string& get_credential(const Vault_credentials &credentials, const Secure_string &key)
+void Vault_credentials::swap(Vault_credentials &obj)
 {
-  Vault_credentials::const_iterator it = credentials.find(key);
-  if (it == credentials.end())
-    return empty_value;
-  else 
-    return it->second;
+  vault_url_.swap(obj.vault_url_);
+  secret_mount_point_.swap(obj.secret_mount_point_);
+  vault_ca_.swap(obj.vault_ca_);
+  token_.swap(obj.token_);
+  std::swap(secret_mount_point_version_, obj.secret_mount_point_version_);
 }
 
-}
 
+}  // namespace keyring
