@@ -1,14 +1,21 @@
 /* Copyright (C) 2007 Google Inc.
-   Copyright (c) 2008, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -16,6 +23,10 @@
 
 
 #include "semisync_master.h"
+#if defined(ENABLED_DEBUG_SYNC)
+#include "debug_sync.h"
+#include "sql_class.h"
+#endif
 
 #define TIME_THOUSAND 1000
 #define TIME_MILLION  1000000
@@ -41,6 +52,7 @@ unsigned long rpl_semi_sync_master_clients          = 0;
 unsigned long long rpl_semi_sync_master_net_wait_time = 0;
 unsigned long long rpl_semi_sync_master_trx_wait_time = 0;
 char rpl_semi_sync_master_wait_no_slave = 1;
+unsigned int rpl_semi_sync_master_wait_for_slave_count= 1;
 
 
 static int getWaitTime(const struct timespec& start_ts);
@@ -51,7 +63,7 @@ static unsigned long long timespec_to_usec(const struct timespec *ts)
   return (unsigned long long) ts->tv_sec * TIME_MILLION + ts->tv_nsec / TIME_THOUSAND;
 #else
   return ts->tv.i64 / 10;
-#endif /* __WIN__ */
+#endif
 }
 
 /*******************************************************************************
@@ -219,6 +231,54 @@ bool ActiveTranx::is_tranx_end_pos(const char *log_file_name,
   return (entry != NULL);
 }
 
+int ActiveTranx::signal_waiting_sessions_all()
+{
+  const char *kWho = "ActiveTranx::signal_waiting_sessions_all";
+  function_enter(kWho);
+  for (TranxNode* entry= trx_front_; entry; entry=entry->next_)
+    mysql_cond_broadcast(&entry->cond);
+
+  return function_exit(kWho, 0);
+}
+
+int ActiveTranx::signal_waiting_sessions_up_to(const char *log_file_name,
+                                               my_off_t log_file_pos)
+{
+  const char *kWho = "ActiveTranx::signal_waiting_sessions_up_to";
+  function_enter(kWho);
+
+  TranxNode* entry= trx_front_;
+  int cmp= ActiveTranx::compare(entry->log_name_, entry->log_pos_, log_file_name, log_file_pos) ;
+  while (entry && cmp <= 0)
+  {
+    mysql_cond_broadcast(&entry->cond);
+    entry= entry->next_;
+    if (entry)
+      cmp= ActiveTranx::compare(entry->log_name_, entry->log_pos_, log_file_name, log_file_pos) ;
+  }
+
+  return function_exit(kWho, (entry != NULL));
+}
+
+TranxNode * ActiveTranx::find_active_tranx_node(const char *log_file_name,
+                                                my_off_t log_file_pos)
+{
+  const char *kWho = "ActiveTranx::find_active_tranx_node";
+  function_enter(kWho);
+
+  TranxNode* entry= trx_front_;
+
+  while (entry)
+  {
+    if (ActiveTranx::compare(log_file_name, log_file_pos, entry->log_name_,
+                             entry->log_pos_) <= 0)
+      break;
+    entry= entry->next_;
+  }
+  function_exit(kWho, 0);
+  return entry;
+}
+
 int ActiveTranx::clear_active_tranx_nodes(const char *log_file_name,
 					  my_off_t log_file_pos)
 {
@@ -233,7 +293,8 @@ int ActiveTranx::clear_active_tranx_nodes(const char *log_file_name,
 
     while (new_front)
     {
-      if (compare(new_front, log_file_name, log_file_pos) > 0)
+      if (compare(new_front, log_file_name, log_file_pos) > 0 ||
+          new_front->n_waiters > 0)
         break;
       new_front = new_front->next_;
     }
@@ -303,6 +364,49 @@ int ActiveTranx::clear_active_tranx_nodes(const char *log_file_name,
 }
 
 
+int ReplSemiSyncMaster::reportReplyPacket(uint32 server_id, const uchar *packet,
+                             ulong packet_len)
+{
+  const char *kWho = "ReplSemiSyncMaster::reportReplyPacket";
+  int result= -1;
+  char log_file_name[FN_REFLEN+1];
+  my_off_t log_file_pos;
+  ulong log_file_len = 0;
+
+  function_enter(kWho);
+
+  if (unlikely(packet[REPLY_MAGIC_NUM_OFFSET] != ReplSemiSyncMaster::kPacketMagicNum))
+  {
+    sql_print_error("Read semi-sync reply magic number error");
+    goto l_end;
+  }
+
+  if (unlikely(packet_len < REPLY_BINLOG_NAME_OFFSET))
+  {
+    sql_print_error("Read semi-sync reply length error: packet is too small");
+    goto l_end;
+  }
+
+  log_file_pos = uint8korr(packet + REPLY_BINLOG_POS_OFFSET);
+  log_file_len = packet_len - REPLY_BINLOG_NAME_OFFSET;
+  if (unlikely(log_file_len >= FN_REFLEN))
+  {
+    sql_print_error("Read semi-sync reply binlog file length too large");
+    goto l_end;
+  }
+  strncpy(log_file_name, (const char*)packet + REPLY_BINLOG_NAME_OFFSET, log_file_len);
+  log_file_name[log_file_len] = 0;
+
+  if (trace_level_ & kTraceDetail)
+    sql_print_information("%s: Got reply(%s, %lu) from server %u",
+                          kWho, log_file_name, (ulong)log_file_pos, server_id);
+
+  handleAck(server_id, log_file_name, log_file_pos);
+
+l_end:
+  return function_exit(kWho, result);
+}
+
 /*******************************************************************************
  *
  * <ReplSemiSyncMaster> class: the basic code layer for sync-replication master.
@@ -348,7 +452,7 @@ int ReplSemiSyncMaster::initObject()
 
   if (init_done_)
   {
-    fprintf(stderr, "%s called twice\n", kWho);
+    sql_print_warning("%s called twice", kWho);
     return 1;
   }
   init_done_ = true;
@@ -360,8 +464,13 @@ int ReplSemiSyncMaster::initObject()
   /* Mutex initialization can only be done after MY_INIT(). */
   mysql_mutex_init(key_ss_mutex_LOCK_binlog_,
                    &LOCK_binlog_, MY_MUTEX_INIT_FAST);
-  mysql_cond_init(key_ss_cond_COND_binlog_send_,
-                  &COND_binlog_send_, NULL);
+
+  /*
+    rpl_semi_sync_master_wait_for_slave_count may be set through mysqld option.
+    So call setWaitSlaveCount to initialize the internal ack container.
+  */
+  if (setWaitSlaveCount(rpl_semi_sync_master_wait_for_slave_count))
+    return 1;
 
   if (rpl_semi_sync_master_enabled)
     result = enableMaster();
@@ -380,7 +489,9 @@ int ReplSemiSyncMaster::enableMaster()
 
   if (!getMasterEnabled())
   {
-    active_tranxs_ = new ActiveTranx(&LOCK_binlog_, trace_level_);
+    if (active_tranxs_ == NULL)
+      active_tranxs_ = new ActiveTranx(&LOCK_binlog_, trace_level_);
+
     if (active_tranxs_ != NULL)
     {
       commit_file_name_inited_ = false;
@@ -388,7 +499,14 @@ int ReplSemiSyncMaster::enableMaster()
       wait_file_name_inited_   = false;
 
       set_master_enabled(true);
-      state_ = true;
+      /*
+        state_ will be set off when users don't want to wait(
+        rpl_semi_sync_master_wait_no_slave == 0) if there is no enough active
+        semisync clients
+      */
+      state_ = (rpl_semi_sync_master_wait_no_slave != 0 ||
+                (rpl_semi_sync_master_clients >=
+                 rpl_semi_sync_master_wait_for_slave_count));
       sql_print_information("Semi-sync replication enabled on the master.");
     }
     else
@@ -415,14 +533,18 @@ int ReplSemiSyncMaster::disableMaster()
      */
     switch_off();
 
-    assert(active_tranxs_ != NULL);
-    delete active_tranxs_;
-    active_tranxs_ = NULL;
+    if ( active_tranxs_ && active_tranxs_->is_empty())
+    {
+      delete active_tranxs_;
+      active_tranxs_ = NULL;
+    }
 
     reply_file_name_inited_ = false;
     wait_file_name_inited_  = false;
     commit_file_name_inited_ = false;
 
+    ack_container_.clear();
+
     set_master_enabled(false);
     sql_print_information("Semi-sync replication disabled on the master.");
   }
@@ -437,7 +559,6 @@ ReplSemiSyncMaster::~ReplSemiSyncMaster()
   if (init_done_)
   {
     mysql_mutex_destroy(&LOCK_binlog_);
-    mysql_cond_destroy(&COND_binlog_send_);
   }
 
   delete active_tranxs_;
@@ -453,22 +574,6 @@ void ReplSemiSyncMaster::unlock()
   mysql_mutex_unlock(&LOCK_binlog_);
 }
 
-void ReplSemiSyncMaster::cond_broadcast()
-{
-  mysql_cond_broadcast(&COND_binlog_send_);
-}
-
-int ReplSemiSyncMaster::cond_timewait(struct timespec *wait_time)
-{
-  const char *kWho = "ReplSemiSyncMaster::cond_timewait()";
-  int wait_res;
-
-  function_enter(kWho);
-  wait_res= mysql_cond_timedwait(&COND_binlog_send_,
-                                 &LOCK_binlog_, wait_time);
-  return function_exit(kWho, wait_res);
-}
-
 void ReplSemiSyncMaster::add_slave()
 {
   lock();
@@ -484,13 +589,29 @@ void ReplSemiSyncMaster::remove_slave()
   /* Only switch off if semi-sync is enabled and is on */
   if (getMasterEnabled() && is_on())
   {
-    /* If user has chosen not to wait if no semi-sync slave available
-       and the last semi-sync slave exits, turn off semi-sync on master
-       immediately.
-     */
-    if (!rpl_semi_sync_master_wait_no_slave &&
-        rpl_semi_sync_master_clients == 0)
+
+    /*
+      If user has chosen not to wait if no enough semi-sync slave available
+      and after a slave exists, turn off semi-semi master immediately if active
+      slaves are less then required slave numbers.
+    */
+    if ((rpl_semi_sync_master_clients ==
+         rpl_semi_sync_master_wait_for_slave_count - 1) &&
+        (!rpl_semi_sync_master_wait_no_slave || abort_loop))
+    {
+      if (abort_loop)
+      {
+        if (commit_file_name_inited_ && reply_file_name_inited_)
+        {
+          int cmp = ActiveTranx::compare(reply_file_name_, reply_file_pos_ ,
+                                         commit_file_name_, commit_file_pos_);
+          if (cmp < 0)
+            sql_print_warning("SEMISYNC: Forced shutdown. Some updates might "
+                              "not be replicated.");
+        }
+      }
       switch_off();
+    }
   }
   unlock();
 }
@@ -503,29 +624,23 @@ bool ReplSemiSyncMaster::is_semi_sync_slave()
   return val;
 }
 
-int ReplSemiSyncMaster::reportReplyBinlog(uint32 server_id,
-					  const char *log_file_name,
-					  my_off_t log_file_pos)
+void ReplSemiSyncMaster::reportReplyBinlog(const char *log_file_name,
+                                           my_off_t log_file_pos)
 {
   const char *kWho = "ReplSemiSyncMaster::reportReplyBinlog";
   int   cmp;
   bool  can_release_threads = false;
   bool  need_copy_send_pos = true;
 
-  if (!(getMasterEnabled()))
-    return 0;
-
   function_enter(kWho);
+  mysql_mutex_assert_owner(&LOCK_binlog_);
 
-  lock();
-
-  /* This is the real check inside the mutex. */
   if (!getMasterEnabled())
     goto l_end;
 
   if (!is_on())
     /* We check to see whether we can switch semi-sync ON. */
-    try_switch_on(server_id, log_file_name, log_file_pos);
+    try_switch_on(log_file_name, log_file_pos);
 
   /* The position should increase monotonically, if there is only one
    * thread sending the binlog to the slave.
@@ -555,14 +670,11 @@ int ReplSemiSyncMaster::reportReplyBinlog(uint32 server_id,
 
   if (need_copy_send_pos)
   {
-    strcpy(reply_file_name_, log_file_name);
+    strncpy(reply_file_name_, log_file_name, sizeof(reply_file_name_) - 1);
+    reply_file_name_[sizeof(reply_file_name_) - 1]= '\0';
     reply_file_pos_ = log_file_pos;
     reply_file_name_inited_ = true;
 
-    /* Remove all active transaction nodes before this point. */
-    assert(active_tranxs_ != NULL);
-    active_tranxs_->clear_active_tranx_nodes(log_file_name, log_file_pos);
-
     if (trace_level_ & kTraceDetail)
       sql_print_information("%s: Got reply at (%s, %lu)", kWho,
                             log_file_name, (unsigned long)log_file_pos);
@@ -586,17 +698,15 @@ int ReplSemiSyncMaster::reportReplyBinlog(uint32 server_id,
   }
 
  l_end:
-  unlock();
 
   if (can_release_threads)
   {
     if (trace_level_ & kTraceDetail)
       sql_print_information("%s: signal all waiting threads.", kWho);
-
-    cond_broadcast();
+    active_tranxs_->signal_waiting_sessions_up_to(reply_file_name_, reply_file_pos_);
   }
 
-  return function_exit(kWho, 0);
+  function_exit(kWho, 0);
 }
 
 int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
@@ -605,23 +715,39 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
   const char *kWho = "ReplSemiSyncMaster::commitTrx";
 
   function_enter(kWho);
+  PSI_stage_info old_stage;
+
+#if defined(ENABLED_DEBUG_SYNC)
+  /* debug sync may not be initialized for a master */
+  if (current_thd->debug_sync_control)
+    DEBUG_SYNC(current_thd, "rpl_semisync_master_commit_trx_before_lock");
+#endif
+  /* Acquire the mutex. */
+  lock();
+
+  TranxNode* entry= NULL;
+  mysql_cond_t* thd_cond= NULL;
+  bool is_semi_sync_trans= true;
+  if (active_tranxs_ != NULL && trx_wait_binlog_name)
+  {
+    entry=
+      active_tranxs_->find_active_tranx_node(trx_wait_binlog_name,
+                                             trx_wait_binlog_pos);
+    if (entry)
+      thd_cond= &entry->cond;
+  }
+  /* This must be called after acquired the lock */
+  THD_ENTER_COND(NULL, thd_cond, &LOCK_binlog_,
+                 & stage_waiting_for_semi_sync_ack_from_slave,
+                 & old_stage);
 
   if (getMasterEnabled() && trx_wait_binlog_name)
   {
     struct timespec start_ts;
     struct timespec abstime;
     int wait_result;
-    const char *old_msg= 0;
-
-    set_timespec(start_ts, 0);
-
-    /* Acquire the mutex. */
-    lock();
-
-    /* This must be called after acquired the lock */
-    old_msg= thd_enter_cond(NULL, &COND_binlog_send_, &LOCK_binlog_,
-                            "Waiting for semi-sync ACK from slave");
 
+    set_timespec(&start_ts, 0);
     /* This is the real check inside the mutex. */
     if (!getMasterEnabled() || !is_on())
       goto l_end;
@@ -633,6 +759,21 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
                             (int)is_on());
     }
 
+    /* Calcuate the waiting period. */
+#ifndef HAVE_STRUCT_TIMESPEC
+      abstime.tv.i64 = start_ts.tv.i64 + (__int64)wait_timeout_ * TIME_THOUSAND * 10;
+      abstime.max_timeout_msec= (long)wait_timeout_;
+#else
+      abstime.tv_sec = start_ts.tv_sec + wait_timeout_ / TIME_THOUSAND;
+      abstime.tv_nsec = start_ts.tv_nsec +
+        (wait_timeout_ % TIME_THOUSAND) * TIME_MILLION;
+      if (abstime.tv_nsec >= TIME_BILLION)
+      {
+        abstime.tv_sec++;
+        abstime.tv_nsec -= TIME_BILLION;
+      }
+#endif /* _WIN32 */
+
     while (is_on())
     {
       if (reply_file_name_inited_)
@@ -650,6 +791,25 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
           break;
         }
       }
+      /*
+        When code reaches here an Entry object may not be present in the
+        following scenario.
+
+        Semi sync was not enabled when transaction entered into ordered_commit
+        process. During flush stage, semi sync was not enabled and there was no
+        'Entry' object created for the transaction being committed and at a
+        later stage it was enabled. In this case trx_wait_binlog_name and
+        trx_wait_binlog_pos are set but the 'Entry' object is not present. Hence
+        dump thread will not wait for reply from slave and it will not update
+        reply_file_name. In such case the committing transaction should not wait
+        for an ack from slave and it should be considered as an async
+        transaction.
+      */
+      if (!entry)
+      {
+        is_semi_sync_trans= false;
+        goto l_end;
+      }
 
       /* Let us update the info about the minimum binlog position of waiting
        * threads.
@@ -661,7 +821,8 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
         if (cmp <= 0)
 	{
           /* This thd has a lower position, let's update the minimum info. */
-          strcpy(wait_file_name_, trx_wait_binlog_name);
+          strncpy(wait_file_name_, trx_wait_binlog_name, sizeof(wait_file_name_) - 1);
+          wait_file_name_[sizeof(wait_file_name_) - 1]= '\0';
           wait_file_pos_ = trx_wait_binlog_pos;
 
           rpl_semi_sync_master_wait_pos_backtraverse++;
@@ -672,7 +833,8 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
       }
       else
       {
-        strcpy(wait_file_name_, trx_wait_binlog_name);
+        strncpy(wait_file_name_, trx_wait_binlog_name, sizeof(wait_file_name_) - 1);
+        wait_file_name_[sizeof(wait_file_name_) - 1]= '\0';
         wait_file_pos_ = trx_wait_binlog_pos;
         wait_file_name_inited_ = true;
 
@@ -681,22 +843,6 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
                                 kWho, wait_file_name_, (unsigned long)wait_file_pos_);
       }
 
-      /* Calcuate the waiting period. */
-#ifndef HAVE_STRUCT_TIMESPEC
-      abstime.tv.i64 = start_ts.tv.i64 + (__int64)wait_timeout_ * TIME_THOUSAND * 10;
-      abstime.max_timeout_msec= (long)wait_timeout_;
-#else
-      unsigned long long diff_nsecs =
-        start_ts.tv_nsec + (unsigned long long)wait_timeout_ * TIME_MILLION;
-      abstime.tv_sec = start_ts.tv_sec;
-      while (diff_nsecs >= TIME_BILLION)
-      {
-        abstime.tv_sec++;
-        diff_nsecs -= TIME_BILLION;
-      }
-      abstime.tv_nsec = diff_nsecs;
-#endif /* __WIN__ */
-      
       /* In semi-synchronous replication, we wait until the binlog-dump
        * thread has received the reply on the relevant binlog segment from the
        * replication slave.
@@ -705,6 +851,15 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
        * when replication has progressed far enough, we will release
        * these waiting threads.
        */
+      if (abort_loop && (rpl_semi_sync_master_clients ==
+                         rpl_semi_sync_master_wait_for_slave_count - 1) && is_on())
+      {
+        sql_print_warning("SEMISYNC: Forced shutdown. Some updates might "
+                          "not be replicated.");
+        switch_off();
+        break;
+      }
+
       rpl_semi_sync_master_wait_sessions++;
       
       if (trace_level_ & kTraceDetail)
@@ -712,8 +867,20 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
                               kWho, wait_timeout_,
                               wait_file_name_, (unsigned long)wait_file_pos_);
       
-      wait_result = cond_timewait(&abstime);
-      rpl_semi_sync_master_wait_sessions--;
+      /* wait for the position to be ACK'ed back */
+      assert(entry);
+      entry->n_waiters++;
+      wait_result= mysql_cond_timedwait(&entry->cond, &LOCK_binlog_, &abstime);
+      entry->n_waiters--;
+      /*
+        After we release LOCK_binlog_ above while waiting for the condition,
+        it can happen that some other parallel client session executed
+        RESET MASTER. That can set rpl_semi_sync_master_wait_sessions to zero.
+        Hence check the value before decrementing it and decrement it only if it is
+        non-zero value.
+      */
+      if (rpl_semi_sync_master_wait_sessions > 0)
+        rpl_semi_sync_master_wait_sessions--;
       
       if (wait_result != 0)
       {
@@ -736,9 +903,10 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
         {
           if (trace_level_ & kTraceGeneral)
           {
-            sql_print_error("Replication semi-sync getWaitTime fail at "
-                            "wait position (%s, %lu)",
-                            trx_wait_binlog_name, (unsigned long)trx_wait_binlog_pos);
+            sql_print_information("Assessment of waiting time for commitTrx "
+                                  "failed at wait position (%s, %lu)",
+                                  trx_wait_binlog_name,
+                                  (unsigned long)trx_wait_binlog_pos);
           }
           rpl_semi_sync_master_timefunc_fails++;
         }
@@ -750,28 +918,46 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
       }
     }
 
-  l_end:
-    /*
-      At this point, the binlog file and position of this transaction
-      must have been removed from ActiveTranx.
-    */
-    assert(!getMasterEnabled() ||
-           !active_tranxs_->is_tranx_end_pos(trx_wait_binlog_name,
-                                             trx_wait_binlog_pos));
-    
+l_end:
     /* Update the status counter. */
-    if (is_on())
+    if (is_on() && is_semi_sync_trans)
       rpl_semi_sync_master_yes_transactions++;
     else
       rpl_semi_sync_master_no_transactions++;
 
-    /* The lock held will be released by thd_exit_cond, so no need to
-       call unlock() here */
-    thd_exit_cond(NULL, old_msg);
   }
 
+  /* Last waiter removes the TranxNode */
+  if (trx_wait_binlog_name && active_tranxs_
+      && entry && entry->n_waiters == 0)
+    active_tranxs_->clear_active_tranx_nodes(trx_wait_binlog_name,
+                                             trx_wait_binlog_pos);
+
+  unlock();
+  THD_EXIT_COND(NULL, & old_stage);
   return function_exit(kWho, 0);
 }
+void ReplSemiSyncMaster::set_wait_no_slave(const void *val)
+{
+  lock();
+  char set_switch= *(char *)val;
+  if (set_switch == 0)
+  {
+    if ((rpl_semi_sync_master_clients == 0) && (is_on()))
+      switch_off();
+  }
+  else
+  {
+    if (!is_on() && getMasterEnabled())
+      force_switch_on();
+  }
+  unlock();
+}
+
+void ReplSemiSyncMaster::force_switch_on()
+{
+  state_= true;
+}
 
 /* Indicate that semi-sync replication is OFF now.
  * 
@@ -786,35 +972,31 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
  *
  * If semi-sync is disabled, all transactions still update the wait
  * position with the last position in binlog.  But no transactions will
- * wait for confirmations and the active transaction list would not be
- * maintained.  In binlog dump thread, updateSyncHeader() checks whether
- * the current sending event catches up with last wait position.  If it
- * does match, semi-sync will be switched on again.
+ * wait for confirmations maintained.  In binlog dump thread,
+ * updateSyncHeader() checks whether the current sending event catches
+ * up with last wait position.  If it does match, semi-sync will be
+ * switched on again.
  */
 int ReplSemiSyncMaster::switch_off()
 {
   const char *kWho = "ReplSemiSyncMaster::switch_off";
-  int result;
 
   function_enter(kWho);
   state_ = false;
 
-  /* Clear the active transaction list. */
-  assert(active_tranxs_ != NULL);
-  result = active_tranxs_->clear_active_tranx_nodes(NULL, 0);
-
   rpl_semi_sync_master_off_times++;
   wait_file_name_inited_   = false;
   reply_file_name_inited_  = false;
   sql_print_information("Semi-sync replication switched OFF.");
-  cond_broadcast();                            /* wake up all waiting threads */
 
-  return function_exit(kWho, result);
+  /* signal waiting sessions */
+  active_tranxs_->signal_waiting_sessions_all();
+
+  return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::try_switch_on(int server_id,
-				      const char *log_file_name,
-				      my_off_t log_file_pos)
+int ReplSemiSyncMaster::try_switch_on(const char *log_file_name,
+                                      my_off_t log_file_pos)
 {
   const char *kWho = "ReplSemiSyncMaster::try_switch_on";
   bool semi_sync_on = false;
@@ -843,10 +1025,8 @@ int ReplSemiSyncMaster::try_switch_on(int server_id,
     /* Switch semi-sync replication on. */
     state_ = true;
 
-    sql_print_information("Semi-sync replication switched ON with slave (server_id: %d) "
-                          "at (%s, %lu)",
-                          server_id, log_file_name,
-                          (unsigned long)log_file_pos);
+    sql_print_information("Semi-sync replication switched ON at (%s, %lu)",
+                          log_file_name, (unsigned long)log_file_pos);
   }
 
   return function_exit(kWho, 0);
@@ -859,11 +1039,6 @@ int ReplSemiSyncMaster::reserveSyncHeader(unsigned char *header,
   function_enter(kWho);
 
   int hlen=0;
-  if (!is_semi_sync_slave())
-  {
-    hlen= 0;
-  }
-  else
   {
     /* No enough space for the extra header, disable semi-sync master */
     if (sizeof(kSyncHeader) > size)
@@ -893,10 +1068,10 @@ int ReplSemiSyncMaster::updateSyncHeader(unsigned char *packet,
   int  cmp = 0;
   bool sync = false;
 
-  /* If the semi-sync master is not enabled, or the slave is not a semi-sync
-   * target, do not request replies from the slave.
+  /* If the semi-sync master is not enabled, do not request replies from the
+     slave.
    */
-  if (!getMasterEnabled() || !is_semi_sync_slave())
+  if (!getMasterEnabled())
     return 0;
 
   function_enter(kWho);
@@ -1043,20 +1218,39 @@ int ReplSemiSyncMaster::writeTranxInBinlog(const char* log_file_name,
   return function_exit(kWho, result);
 }
 
+int ReplSemiSyncMaster::skipSlaveReply(const char *event_buf,
+                                       uint32 server_id,
+                                       const char* skipped_log_file,
+                                       my_off_t skipped_log_pos)
+{
+  const char *kWho = "ReplSemiSyncMaster::skipSlaveReply";
+
+  function_enter(kWho);
+
+  assert((unsigned char)event_buf[1] == kPacketMagicNum);
+  if ((unsigned char)event_buf[2] != kPacketFlagSync)
+  {
+    /* current event would not require a reply anyway */
+    goto l_end;
+  }
+
+  if (trace_level_ & kTraceDetail)
+    sql_print_information("%s: Transaction skipped at (%s, %lu)", kWho,
+                          skipped_log_file, (unsigned long)skipped_log_pos);
+
+  /* Treat skipped event as a received ack */
+  handleAck(server_id, skipped_log_file, skipped_log_pos);
+
+ l_end:
+  return function_exit(kWho, 0);
+}
+
 int ReplSemiSyncMaster::readSlaveReply(NET *net, uint32 server_id,
                                        const char *event_buf)
 {
   const char *kWho = "ReplSemiSyncMaster::readSlaveReply";
-  const unsigned char *packet;
-  char     log_file_name[FN_REFLEN];
-  my_off_t log_file_pos;
-  ulong    log_file_len = 0;
-  ulong    packet_len;
   int      result = -1;
 
-  struct timespec start_ts= { 0, 0 };
-  ulong trc_level = trace_level_;
-
   function_enter(kWho);
 
   assert((unsigned char)event_buf[1] == kPacketMagicNum);
@@ -1067,9 +1261,6 @@ int ReplSemiSyncMaster::readSlaveReply(NET *net, uint32 server_id,
     goto l_end;
   }
 
-  if (trc_level & kTraceNetWait)
-    set_timespec(start_ts, 0);
-
   /* We flush to make sure that the current event is sent to the network,
    * instead of being buffered in the TCP/IP stack.
    */
@@ -1081,65 +1272,9 @@ int ReplSemiSyncMaster::readSlaveReply(NET *net, uint32 server_id,
   }
 
   net_clear(net, 0);
-  if (trc_level & kTraceDetail)
-    sql_print_information("%s: Wait for replica's reply", kWho);
-
-  /* Wait for the network here.  Though binlog dump thread can indefinitely wait
-   * here, transactions would not wait indefintely.
-   * Transactions wait on binlog replies detected by binlog dump threads.  If
-   * binlog dump threads wait too long, transactions will timeout and continue.
-   */
-  packet_len = my_net_read(net);
-
-  if (trc_level & kTraceNetWait)
-  {
-    int wait_time = getWaitTime(start_ts);
-    if (wait_time < 0)
-    {
-      sql_print_error("Semi-sync master wait for reply "
-                      "fail to get wait time.");
-      rpl_semi_sync_master_timefunc_fails++;
-    }
-    else
-    {
-      rpl_semi_sync_master_net_wait_num++;
-      rpl_semi_sync_master_net_wait_time += wait_time;
-    }
-  }
-
-  if (packet_len == packet_error || packet_len < REPLY_BINLOG_NAME_OFFSET)
-  {
-    if (packet_len == packet_error)
-      sql_print_error("Read semi-sync reply network error: %s (errno: %d)",
-                      net->last_error, net->last_errno);
-    else
-      sql_print_error("Read semi-sync reply length error: %s (errno: %d)",
-                      net->last_error, net->last_errno);
-    goto l_end;
-  }
-
-  packet = net->read_pos;
-  if (packet[REPLY_MAGIC_NUM_OFFSET] != ReplSemiSyncMaster::kPacketMagicNum)
-  {
-    sql_print_error("Read semi-sync reply magic number error");
-    goto l_end;
-  }
-
-  log_file_pos = uint8korr(packet + REPLY_BINLOG_POS_OFFSET);
-  log_file_len = packet_len - REPLY_BINLOG_NAME_OFFSET;
-  if (log_file_len >= FN_REFLEN)
-  {
-    sql_print_error("Read semi-sync reply binlog file length too large");
-    goto l_end;
-  }
-  strncpy(log_file_name, (const char*)packet + REPLY_BINLOG_NAME_OFFSET, log_file_len);
-  log_file_name[log_file_len] = 0;
-
-  if (trc_level & kTraceDetail)
-    sql_print_information("%s: Got reply (%s, %lu)",
-                          kWho, log_file_name, (ulong)log_file_pos);
-
-  result = reportReplyBinlog(server_id, log_file_name, log_file_pos);
+  net->pkt_nr++;
+  result = 0;
+  rpl_semi_sync_master_net_wait_num++;
 
  l_end:
   return function_exit(kWho, result);
@@ -1156,7 +1291,7 @@ int ReplSemiSyncMaster::resetMaster()
 
   lock();
 
-  state_ = getMasterEnabled()? 1 : 0;
+  ack_container_.clear();
 
   wait_file_name_inited_   = false;
   reply_file_name_inited_  = false;
@@ -1195,6 +1330,122 @@ void ReplSemiSyncMaster::setExportStats()
   unlock();
 }
 
+int ReplSemiSyncMaster::setWaitSlaveCount(unsigned int new_value)
+{
+  const AckInfo *ackinfo= NULL;
+  int result= 0;
+
+  const char *kWho = "ReplSemiSyncMaster::updateWaitSlaves";
+  function_enter(kWho);
+
+  lock();
+
+  result= ack_container_.resize(new_value, &ackinfo);
+  if (result == 0)
+  {
+    rpl_semi_sync_master_wait_for_slave_count= new_value;
+    if (ackinfo != NULL)
+      reportReplyBinlog(ackinfo->binlog_name, ackinfo->binlog_pos);
+  }
+
+  unlock();
+  return function_exit(kWho, result);
+}
+
+const AckInfo* AckContainer::insert(int server_id, const char *log_file_name,
+                                    my_off_t log_file_pos)
+{
+  const AckInfo *ret_ack= NULL;
+
+  const char *kWho = "AckContainer::insert";
+  function_enter(kWho);
+
+  if (!m_greatest_ack.less_than(log_file_name, log_file_pos))
+  {
+    if (trace_level_ & kTraceDetail)
+      sql_print_information("The received ack is smaller than m_greatest_ack");
+
+    goto l_end;
+  }
+
+  /* Update the slave's ack position if it is in the ack array */
+  if (updateIfExist(server_id, log_file_name, log_file_pos) < m_size)
+    goto l_end;
+
+  if (full())
+  {
+    AckInfo *min_ack;
+
+    ret_ack= &m_greatest_ack;
+
+    /* Find the minimum ack which is smaller than the inserted ack. */
+    min_ack= minAck(log_file_name, log_file_pos);
+    if (likely(min_ack == NULL))
+    {
+      m_greatest_ack.set(server_id, log_file_name, log_file_pos);
+
+      /* Remove all slaves which have minimum ack position from the ack array */
+      remove_all(log_file_name, log_file_pos);
+
+      /* Don't insert current ack into container if it is the minimum ack. */
+      goto l_end;
+    }
+    else
+    {
+      m_greatest_ack= *min_ack;
+      remove_all(m_greatest_ack.binlog_name, m_greatest_ack.binlog_pos);
+    }
+  }
+
+  m_ack_array[m_empty_slot].set(server_id, log_file_name, log_file_pos);
+
+  if (trace_level_ & kTraceDetail)
+    sql_print_information("Add the ack into slot %u", m_empty_slot);
+
+l_end:
+  function_exit(kWho, 0);
+  return ret_ack;
+}
+
+int AckContainer::resize(unsigned int size, const AckInfo **ackinfo)
+{
+  AckInfo *old_ack_array= m_ack_array;
+  unsigned int old_array_size= m_size;
+  unsigned int i;
+
+  if (size - 1 == m_size)
+    return 0;
+
+  m_size= size - 1;
+  m_ack_array= NULL;
+  if (m_size)
+  {
+    m_ack_array= (AckInfo *)
+      DBUG_EVALUATE_IF("rpl_semisync_simulate_allocate_ack_container_failure",
+                       NULL, my_malloc(0, sizeof(AckInfo) * (size - 1),
+                                       MYF(MY_ZEROFILL)));
+    if (m_ack_array == NULL)
+    {
+      m_ack_array= old_ack_array;
+      m_size= old_array_size;
+      return -1;
+    }
+  }
+
+  if (old_ack_array != NULL)
+  {
+    for (i= 0; i < old_array_size; i++)
+    {
+      const AckInfo *ack= insert(old_ack_array[i]);
+      if (ack)
+        *ackinfo= ack;
+    }
+    my_free(old_ack_array);
+  }
+  return 0;
+}
+
+
 /* Get the waiting time given the wait's staring time.
  * 
  * Return:
@@ -1210,7 +1461,7 @@ static int getWaitTime(const struct timespec& start_ts)
   start_usecs = timespec_to_usec(&start_ts);
 
   /* Get the wait time interval. */
-  set_timespec(end_ts, 0);
+  set_timespec(&end_ts, 0);
 
   /* Ending time in microseconds(us). */
   end_usecs = timespec_to_usec(&end_ts);
