@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 36fea60acf42628575109537cd69588517c02a63 $"
+#ident "$Id: 81a9b451bc28c08df284d14933d0cfdb4822c7bb $"
 
 #include "backup_debug.h"
 #include "check.h"
@@ -58,7 +58,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 template class std::vector<char *>;
 
-#if DEBUG_HOTBACKUP
+#if defined(DEBUG_HOTBACKUP) && DEBUG_HOTBACKUP
 #define WARN(string, arg) HotBackup::CopyWarn(string, arg)
 #define TRACE(string, arg) HotBackup::CopyTrace(string, arg)
 #define ERROR(string, arg) HotBackup::CopyError(string, arg)
@@ -68,7 +68,7 @@ template class std::vector<char *>;
 #define ERROR(string, arg) 
 #endif
 
-#if PAUSE_POINTS_ON
+#if defined(PAUSE_POINTS_ON)
 #define PAUSE(int) while(HotBackup::should_pause(int)) { sched_yield(); }
 #else
 #define PAUSE(int)
@@ -150,7 +150,7 @@ int copier::do_copy(void) throw() {
         
         {
             with_mutex_locked tm(&m_todo_mutex, BACKTRACE(NULL));
-            fname = m_todo.back();
+            fname = m_todo.front();
         }
         TRACE("Copying: ", fname);
         
@@ -166,7 +166,7 @@ int copier::do_copy(void) throw() {
 
         {
             with_mutex_locked tm(&m_todo_mutex, BACKTRACE(NULL));
-            m_todo.pop_back();
+            m_todo.pop_front();
         }
         
         r = this->copy_stripped_file(fname);
