@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -301,7 +301,10 @@ int group_replication_trans_before_commit(Trans_param *param)
     DBUG_RETURN(0);
   }
 
-  shared_plugin_stop_lock->grab_read_lock();
+  if (shared_plugin_stop_lock->try_grab_read_lock()) {
+    /* If plugin is stopping, rollback the transaction immediatly. */
+    DBUG_RETURN(1);
+  }
 
   if (is_plugin_waiting_to_set_server_read_mode())
   {
@@ -837,15 +840,23 @@ Transaction_Message::append_cache(IO_CACHE *src)
     length= my_b_fill(src);
   }
 
-  while (length > 0 && !src->error)
+  try
   {
-    data.insert(data.end(),
-                buffer,
-                buffer + length);
+    while (length > 0 && !src->error)
+    {
+      data.insert(data.end(),
+                  buffer,
+                  buffer + length);
 
-    src->read_pos= src->read_end;
-    length= my_b_fill(src);
-    buffer= src->read_pos;
+      src->read_pos= src->read_end;
+      length= my_b_fill(src);
+      buffer= src->read_pos;
+    }
+  }
+  catch (const std::bad_alloc &)
+  {
+    log_message(MY_ERROR_LEVEL, ER_DEFAULT(ER_OUT_OF_RESOURCES));
+    DBUG_RETURN(true);
   }
 
   DBUG_RETURN(src->error ? true : false);
