@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -93,6 +93,9 @@ public:
     @return Error status.
   */
   virtual bool execute(THD *thd, uint *nextp) = 0;
+#ifdef HAVE_PSI_INTERFACE
+  virtual PSI_statement_info* get_psi_info() = 0;
+#endif
 
   uint get_ip() const
   { return m_ip; }
@@ -108,10 +111,22 @@ public:
   sp_pcontext *get_parsing_ctx() const
   { return m_parsing_ctx; }
 
+protected:
+  /**
+    Clear diagnostics area.
+    @param thd         Thread context
+  */
+  void clear_da(THD *thd) const
+  {
+    thd->get_stmt_da()->reset_diagnostics_area();
+    thd->get_stmt_da()->reset_condition_info(thd);
+  }
+
   ///////////////////////////////////////////////////////////////////////////
   // The following operations are used solely for SP-code-optimizer.
   ///////////////////////////////////////////////////////////////////////////
 
+public:
   /**
     Mark this instruction as reachable during optimization and return the
     index to the next instruction. Jump instruction will add their
@@ -231,7 +246,7 @@ public:
       return m_lex ? m_lex->sql_command : -1;
   }
 
-  const LEX_STRING *get_prepared_stmt_name() const {
+  const LEX_CSTRING *get_prepared_stmt_name() const {
     return m_lex ? &m_lex->prepared_stmt_name : NULL;
   }
 
@@ -290,7 +305,15 @@ public:
   /////////////////////////////////////////////////////////////////////////
 
   virtual bool execute(THD *thd, uint *nextp)
-  { return validate_lex_and_execute_core(thd, nextp, true); }
+  {
+    /*
+      SP instructions with expressions should clear DA before execution.
+      Note that sp_instr_stmt will override execute(), but it clears DA
+      during normal mysql_execute_command().
+    */
+    clear_da(thd);
+    return validate_lex_and_execute_core(thd, nextp, true);
+  }
 
 protected:
   /////////////////////////////////////////////////////////////////////////
@@ -472,12 +495,22 @@ private:
 
   /// Specify if the stored LEX-object is up-to-date.
   bool m_valid;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
 
 /**
-  sp_instr_set represents SET-statememnts, which deal with SP-variables.
+  sp_instr_set represents SET-statements, which deal with SP-variables.
 */
 class sp_instr_set : public sp_lex_instr
 {
@@ -514,9 +547,9 @@ public:
 
   virtual bool on_after_expr_parsing(THD *thd)
   {
-    DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+    assert(thd->lex->select_lex->item_list.elements == 1);
 
-    m_value_item= thd->lex->select_lex.item_list.head();
+    m_value_item= thd->lex->select_lex->item_list.head();
 
     return false;
   }
@@ -533,6 +566,15 @@ private:
 
   /// SQL-query corresponding to the value expression.
   LEX_STRING m_value_query;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  static PSI_statement_info psi_info;
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -594,6 +636,16 @@ private:
 
   /// SQL-query corresponding to the value expression.
   LEX_STRING m_value_query;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -648,9 +700,9 @@ public:
 
   virtual bool on_after_expr_parsing(THD *thd)
   {
-    DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+    assert(thd->lex->select_lex->item_list.elements == 1);
 
-    m_expr_item= thd->lex->select_lex.item_list.head();
+    m_expr_item= thd->lex->select_lex->item_list.head();
 
     return false;
   }
@@ -667,6 +719,16 @@ private:
 
   /// RETURN-field type code.
   enum enum_field_types m_return_field_type;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -732,7 +794,7 @@ public:
   virtual void backpatch(uint dest)
   {
     /* Calling backpatch twice is a logic flaw in jump resolution. */
-    DBUG_ASSERT(m_dest == 0);
+    assert(m_dest == 0);
     m_dest= dest;
   }
 
@@ -742,6 +804,16 @@ protected:
 
   // The following attribute is used by SP-optimizer.
   sp_instr *m_optdest;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -821,7 +893,7 @@ public:
   virtual void backpatch(uint dest)
   {
     /* Calling backpatch twice is a logic flaw in jump resolution. */
-    DBUG_ASSERT(m_dest == 0);
+    assert(m_dest == 0);
     m_dest= dest;
   }
 
@@ -883,12 +955,22 @@ public:
 
   virtual bool on_after_expr_parsing(THD *thd)
   {
-    DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+    assert(thd->lex->select_lex->item_list.elements == 1);
 
-    m_expr_item= thd->lex->select_lex.item_list.head();
+    m_expr_item= thd->lex->select_lex->item_list.head();
 
     return false;
   }
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -960,9 +1042,9 @@ public:
 
   virtual bool on_after_expr_parsing(THD *thd)
   {
-    DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+    assert(thd->lex->select_lex->item_list.elements == 1);
 
-    m_expr_item= thd->lex->select_lex.item_list.head();
+    m_expr_item= thd->lex->select_lex->item_list.head();
 
     return false;
   }
@@ -970,6 +1052,16 @@ public:
 private:
   /// Identifier (index) of the CASE-expression in the runtime context.
   uint m_case_expr_id;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1053,6 +1145,16 @@ private:
     expression.
   */
   Item *m_eq_item;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1070,7 +1172,7 @@ public:
     m_opt_hpop(0),
     m_frame(ctx->current_var_count())
   {
-    DBUG_ASSERT(m_handler->condition_values.elements == 0);
+    assert(m_handler->condition_values.elements == 0);
   }
 
   virtual ~sp_instr_hpush_jump()
@@ -1109,7 +1211,7 @@ public:
 
   virtual void backpatch(uint dest)
   {
-    DBUG_ASSERT(!m_dest || !m_opt_hpop);
+    assert(!m_dest || !m_opt_hpop);
     if (!m_dest)
       m_dest= dest;
     else
@@ -1126,6 +1228,16 @@ private:
   // This attribute is needed for SHOW PROCEDURE CODE only (i.e. it's needed in
   // debug version only). It's used in print().
   uint m_frame;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1149,6 +1261,16 @@ public:
   /////////////////////////////////////////////////////////////////////////
 
   virtual bool execute(THD *thd, uint *nextp);
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1183,6 +1305,16 @@ private:
   // This attribute is needed for SHOW PROCEDURE CODE only (i.e. it's needed in
   // debug version only). It's used in print().
   uint m_frame;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1280,6 +1412,16 @@ private:
 
   /// Used to identify the cursor in the sp_rcontext.
   int m_cursor_idx;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1310,6 +1452,16 @@ public:
 
 private:
   uint m_count;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1341,6 +1493,16 @@ public:
 private:
   /// Used to identify the cursor in the sp_rcontext.
   int m_cursor_idx;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1373,6 +1535,16 @@ public:
 private:
   /// Used to identify the cursor in the sp_rcontext.
   int m_cursor_idx;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1411,6 +1583,16 @@ private:
 
   /// Used to identify the cursor in the sp_rcontext.
   int m_cursor_idx;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1455,6 +1637,16 @@ public:
 private:
   /// The error code, which should be raised by this instruction.
   int m_errcode;
+
+#ifdef HAVE_PSI_INTERFACE
+public:
+  virtual PSI_statement_info* get_psi_info()
+  {
+    return & psi_info;
+  }
+
+  static PSI_statement_info psi_info;
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////
