@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -377,8 +385,8 @@ private:
 	/** Wakeup a waiting thread */
 	void signal() UNIV_NOTHROW
 	{
-		syscall(SYS_futex, &m_lock_word, FUTEX_WAKE_PRIVATE,
-			MUTEX_STATE_LOCKED, 0, 0, 0);
+		syscall(SYS_futex, &m_lock_word, FUTEX_WAKE_PRIVATE, 1, 0, 0,
+			0);
 	}
 
 	/** Poll waiting for mutex to be unlocked.
@@ -637,11 +645,8 @@ struct TTASEventMutex {
 		uint32_t	line)
 		UNIV_NOTHROW
 	{
-		ut_a(m_event == 0);
 		ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
 
-		m_event = os_event_create(sync_latch_get_name(id));
-
 		m_policy.init(*this, id, filename, line);
 	}
 
@@ -653,10 +658,6 @@ struct TTASEventMutex {
 	{
 		ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED);
 
-		/* We have to free the event before InnoDB shuts down. */
-		os_event_destroy(m_event);
-		m_event = 0;
-
 		m_policy.destroy();
 	}
 
@@ -715,7 +716,7 @@ struct TTASEventMutex {
 	os_event_t event()
 		UNIV_NOTHROW
 	{
-		return(m_event);
+		return(&m_event);
 	}
 
 	/** @return true if locked by some thread */
@@ -909,7 +910,7 @@ private:
 	lock_word_t		m_waiters;
 
 	/** Used by sync0arr.cc for the wait queue */
-	os_event_t		m_event;
+	struct os_event		m_event;
 
 	/** Policy data */
 	MutexPolicy		m_policy;
@@ -1076,7 +1077,7 @@ struct PolicyMutex
 	void pfs_add(mysql_pfs_key_t key) UNIV_NOTHROW
 	{
 		ut_ad(m_ptr == 0);
-		m_ptr = PSI_MUTEX_CALL(init_mutex)(key, this);
+		m_ptr = PSI_MUTEX_CALL(init_mutex)(key.m_value, this);
 	}
 
 private:
