@@ -394,7 +394,7 @@ trx_print_low(
 			/*!< in: transaction */
 	ulint		max_query_len,
 			/*!< in: max query length to print,
-			or 0 to use the default max length */
+			must be positive */ 
 	ulint		n_rec_locks,
 			/*!< in: lock_number_of_rows_locked(&trx->lock) */
 	ulint		n_trx_locks,
@@ -412,7 +412,7 @@ trx_print_latched(
 	FILE*		f,		/*!< in: output stream */
 	const trx_t*	trx,		/*!< in: transaction */
 	ulint		max_query_len);	/*!< in: max query length to print,
-					or 0 to use the default max length */
+					  must be positive */
 
 /**********************************************************************//**
 Prints info about a transaction.
@@ -423,7 +423,7 @@ trx_print(
 	FILE*		f,		/*!< in: output stream */
 	const trx_t*	trx,		/*!< in: transaction */
 	ulint		max_query_len);	/*!< in: max query length to print,
-					or 0 to use the default max length */
+					 must be positive */
 
 /**********************************************************************//**
 Determine if a transaction is a dictionary operation.
