@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1997, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -480,7 +480,8 @@ row_vers_non_vc_match(
 @param[in]	clust_index	clustered index
 @param[in]	index		the secondary index
 @param[in]	heap		heap used to build virtual dtuple
-@param[in]	prebuilt	compress_heap must be taken from here */
+@param[in]	prebuilt	provides pointer to blob_heap (used for decompression)
+                        and compress_heap (used for compression) */
 static
 void
 row_vers_build_clust_v_col(
@@ -749,6 +750,20 @@ row_vers_vc_matches_cluster(
 					field2->len = ind_field->prefix_len;
 				}
 
+				/* For multi-byte character sets (like utf8mb4)
+				and index on prefix of varchar vcol, we log
+				prefix_len * mbmaxlen bytes but the actual
+				secondaary index record size can be less than
+				that. For comparision, use actual length of
+				secondary index record */
+				uint8_t mbmax_len =
+				    DATA_MBMAXLEN(field2->type.mbminmaxlen);
+				if (ind_field->prefix_len != 0 &&
+				    !dfield_is_null(field2) &&
+				    mbmax_len > 1) {
+					field2->len = field1->len;
+				}
+
 				/* The index field mismatch */
 				if (v_heap
 				    || cmp_dfield_dfield(field2, field1) != 0) {
@@ -802,7 +817,8 @@ func_exit:
 @param[in,out]	heap		heap memory
 @param[in,out]	v_heap		heap memory to keep virtual colum dtuple
 @param[in]	mtr		mtr holding the latch on rec
-@param[in]	prebuilt	compress_heap must be taken from here
+@param[in]	prebuilt	provides pointer to blob_heap (used for decompression)
+                        and compress_heap (used for compression)
 @return dtuple contains virtual column data */
 static
 const dtuple_t*
@@ -925,11 +941,11 @@ row_vers_old_has_index_entry(
 				NULL, NULL, NULL, &ext, heap);
 
 		if (dict_index_has_virtual(index)) {
-#ifdef DBUG_OFF
+#ifdef NDEBUG
 # define dbug_v_purge false
-#else /* DBUG_OFF */
+#else /* NDEBUG */
                         bool    dbug_v_purge = false;
-#endif /* DBUG_OFF */
+#endif /* NDEBUG */
 
 			DBUG_EXECUTE_IF(
 				"ib_purge_virtual_index_callback",
