@@ -1,6 +1,6 @@
 /***************************************************************************//**
 
-Copyright (c) 2007, 2010, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2007, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -31,6 +31,9 @@ Red-Black tree implementation
 Created 2007-03-20 Sunny Bains
 ***********************************************************************/
 
+#include "univ.i"
+
+#include "ut0new.h"
 #include "ut0rbt.h"
 
 /**********************************************************************//**
@@ -78,7 +81,7 @@ rbt_print_subtree(
 
 /**********************************************************************//**
 Verify that the keys are in order.
-@return	TRUE of OK. FALSE if not ordered */
+@return TRUE of OK. FALSE if not ordered */
 static
 ibool
 rbt_check_ordering(
@@ -117,7 +120,7 @@ rbt_check_ordering(
 /**********************************************************************//**
 Check that every path from the root to the leaves has the same count.
 Count is expressed in the number of black nodes.
-@return	0 on failure else black height of the subtree */
+@return 0 on failure else black height of the subtree */
 static
 ibool
 rbt_count_black_nodes(
@@ -395,7 +398,7 @@ rbt_balance_tree(
 
 /**********************************************************************//**
 Find the given node's successor.
-@return	successor node or NULL if no successor */
+@return successor node or NULL if no successor */
 static
 ib_rbt_node_t*
 rbt_find_successor(
@@ -435,7 +438,7 @@ rbt_find_successor(
 
 /**********************************************************************//**
 Find the given node's precedecessor.
-@return	predecessor node or NULL if no predecesor */
+@return predecessor node or NULL if no predecesor */
 static
 ib_rbt_node_t*
 rbt_find_predecessor(
@@ -526,7 +529,7 @@ rbt_replace_node(
 
 /**********************************************************************//**
 Detach node from the tree replacing it with one of it's children.
-@return	the child node that now occupies the position of the detached node */
+@return the child node that now occupies the position of the detached node */
 static
 ib_rbt_node_t*
 rbt_detach_node(
@@ -569,7 +572,7 @@ rbt_detach_node(
 
 /**********************************************************************//**
 Rebalance the right sub-tree after deletion.
-@return	node to rebalance if more rebalancing required else NULL */
+@return node to rebalance if more rebalancing required else NULL */
 static
 ib_rbt_node_t*
 rbt_balance_right(
@@ -629,7 +632,7 @@ rbt_balance_right(
 
 /**********************************************************************//**
 Rebalance the left sub-tree after deletion.
-@return	node to rebalance if more rebalancing required else NULL */
+@return node to rebalance if more rebalancing required else NULL */
 static
 ib_rbt_node_t*
 rbt_balance_left(
@@ -758,7 +761,6 @@ rbt_free_node(
 
 /**********************************************************************//**
 Free all the nodes and free the tree. */
-UNIV_INTERN
 void
 rbt_free(
 /*=====*/
@@ -772,8 +774,7 @@ rbt_free(
 /**********************************************************************//**
 Create an instance of a red black tree, whose comparison function takes
 an argument
-@return	an empty rb tree */
-UNIV_INTERN
+@return an empty rb tree */
 ib_rbt_t*
 rbt_create_arg_cmp(
 /*===============*/
@@ -795,8 +796,7 @@ rbt_create_arg_cmp(
 
 /**********************************************************************//**
 Create an instance of a red black tree.
-@return	an empty rb tree */
-UNIV_INTERN
+@return an empty rb tree */
 ib_rbt_t*
 rbt_create(
 /*=======*/
@@ -806,22 +806,19 @@ rbt_create(
 	ib_rbt_t*	tree;
 	ib_rbt_node_t*	node;
 
-	tree = (ib_rbt_t*) ut_malloc(sizeof(*tree));
-	memset(tree, 0, sizeof(*tree));
+	tree = (ib_rbt_t*) ut_zalloc_nokey(sizeof(*tree));
 
 	tree->sizeof_value = sizeof_value;
 
 	/* Create the sentinel (NIL) node. */
-	node = tree->nil = (ib_rbt_node_t*) ut_malloc(sizeof(*node));
-	memset(node, 0, sizeof(*node));
+	node = tree->nil = (ib_rbt_node_t*) ut_zalloc_nokey(sizeof(*node));
 
 	node->color = IB_RBT_BLACK;
 	node->parent = node->left = node->right = node;
 
 	/* Create the "fake" root, the real root node will be the
 	left child of this node. */
-	node = tree->root = (ib_rbt_node_t*) ut_malloc(sizeof(*node));
-	memset(node, 0, sizeof(*node));
+	node = tree->root = (ib_rbt_node_t*) ut_zalloc_nokey(sizeof(*node));
 
 	node->color = IB_RBT_BLACK;
 	node->parent = node->left = node->right = tree->nil;
@@ -833,8 +830,7 @@ rbt_create(
 
 /**********************************************************************//**
 Generic insert of a value in the rb tree.
-@return	inserted node */
-UNIV_INTERN
+@return inserted node */
 const ib_rbt_node_t*
 rbt_insert(
 /*=======*/
@@ -846,7 +842,7 @@ rbt_insert(
 	ib_rbt_node_t*	node;
 
 	/* Create the node that will hold the value data. */
-	node = (ib_rbt_node_t*) ut_malloc(SIZEOF_NODE(tree));
+	node = (ib_rbt_node_t*) ut_malloc_nokey(SIZEOF_NODE(tree));
 
 	memcpy(node->value, value, tree->sizeof_value);
 	node->parent = node->left = node->right = tree->nil;
@@ -862,8 +858,7 @@ rbt_insert(
 
 /**********************************************************************//**
 Add a new node to the tree, useful for data that is pre-sorted.
-@return	appended node */
-UNIV_INTERN
+@return appended node */
 const ib_rbt_node_t*
 rbt_add_node(
 /*=========*/
@@ -875,7 +870,7 @@ rbt_add_node(
 	ib_rbt_node_t*	node;
 
 	/* Create the node that will hold the value data */
-	node = (ib_rbt_node_t*) ut_malloc(SIZEOF_NODE(tree));
+	node = (ib_rbt_node_t*) ut_malloc_nokey(SIZEOF_NODE(tree));
 
 	memcpy(node->value, value, tree->sizeof_value);
 	return(rbt_add_preallocated_node(tree, parent, node));
@@ -885,7 +880,7 @@ rbt_add_node(
 Add a new caller-provided node to tree at the specified position.
 The node must have its key fields initialized correctly.
 @return added node */
-UNIV_INTERN
+
 const ib_rbt_node_t*
 rbt_add_preallocated_node(
 /*======================*/
@@ -916,8 +911,7 @@ rbt_add_preallocated_node(
 
 /**********************************************************************//**
 Find a matching node in the rb tree.
-@return	NULL if not found else the node where key was found */
-UNIV_INTERN
+@return NULL if not found else the node where key was found */
 const ib_rbt_node_t*
 rbt_lookup(
 /*=======*/
@@ -951,8 +945,7 @@ rbt_lookup(
 
 /**********************************************************************//**
 Delete a node indentified by key.
-@return	TRUE if success FALSE if not found */
-UNIV_INTERN
+@return TRUE if success FALSE if not found */
 ibool
 rbt_delete(
 /*=======*/
@@ -975,8 +968,7 @@ rbt_delete(
 /**********************************************************************//**
 Remove a node from the rb tree, the node is not free'd, that is the
 callers responsibility.
-@return	deleted node but without the const */
-UNIV_INTERN
+@return deleted node but without the const */
 ib_rbt_node_t*
 rbt_remove_node(
 /*============*/
@@ -998,8 +990,7 @@ rbt_remove_node(
 
 /**********************************************************************//**
 Find the node that has the lowest key that is >= key.
-@return	node satisfying the lower bound constraint or NULL */
-UNIV_INTERN
+@return node satisfying the lower bound constraint or NULL */
 const ib_rbt_node_t*
 rbt_lower_bound(
 /*============*/
@@ -1039,8 +1030,7 @@ rbt_lower_bound(
 
 /**********************************************************************//**
 Find the node that has the greatest key that is <= key.
-@return	node satisfying the upper bound constraint or NULL */
-UNIV_INTERN
+@return node satisfying the upper bound constraint or NULL */
 const ib_rbt_node_t*
 rbt_upper_bound(
 /*============*/
@@ -1080,8 +1070,7 @@ rbt_upper_bound(
 
 /**********************************************************************//**
 Find the node that has the greatest key that is <= key.
-@return	value of result */
-UNIV_INTERN
+@return value of result */
 int
 rbt_search(
 /*=======*/
@@ -1121,8 +1110,7 @@ rbt_search(
 /**********************************************************************//**
 Find the node that has the greatest key that is <= key. But use the
 supplied comparison function.
-@return	value of result */
-UNIV_INTERN
+@return value of result */
 int
 rbt_search_cmp(
 /*===========*/
@@ -1166,7 +1154,6 @@ rbt_search_cmp(
 
 /**********************************************************************//**
 Return the left most node in the tree. */
-UNIV_INTERN
 const ib_rbt_node_t*
 rbt_first(
 /*======*/
@@ -1186,8 +1173,7 @@ rbt_first(
 
 /**********************************************************************//**
 Return the right most node in the tree.
-@return	the rightmost node or NULL */
-UNIV_INTERN
+@return the rightmost node or NULL */
 const ib_rbt_node_t*
 rbt_last(
 /*=====*/
@@ -1206,8 +1192,7 @@ rbt_last(
 
 /**********************************************************************//**
 Return the next node.
-@return	node next from current */
-UNIV_INTERN
+@return node next from current */
 const ib_rbt_node_t*
 rbt_next(
 /*=====*/
@@ -1219,8 +1204,7 @@ rbt_next(
 
 /**********************************************************************//**
 Return the previous node.
-@return	node prev from current */
-UNIV_INTERN
+@return node prev from current */
 const ib_rbt_node_t*
 rbt_prev(
 /*=====*/
@@ -1232,7 +1216,6 @@ rbt_prev(
 
 /**********************************************************************//**
 Reset the tree. Delete all the nodes. */
-UNIV_INTERN
 void
 rbt_clear(
 /*======*/
@@ -1244,7 +1227,7 @@ rbt_clear(
 
 /****************************************************************//**
 Clear the tree without deleting and freeing its nodes. */
-UNIV_INTERN
+
 void
 rbt_reset(
 /*======*/
@@ -1256,8 +1239,7 @@ rbt_reset(
 
 /**********************************************************************//**
 Merge the node from dst into src. Return the number of nodes merged.
-@return	no. of recs merged */
-UNIV_INTERN
+@return no. of recs merged */
 ulint
 rbt_merge_uniq(
 /*===========*/
@@ -1287,8 +1269,7 @@ rbt_merge_uniq(
 Merge the node from dst into src. Return the number of nodes merged.
 Delete the nodes from src after copying node to dst. As a side effect
 the duplicates will be left untouched in the src.
-@return	no. of recs merged */
-UNIV_INTERN
+@return no. of recs merged */
 ulint
 rbt_merge_uniq_destructive(
 /*=======================*/
@@ -1334,8 +1315,7 @@ rbt_merge_uniq_destructive(
 /**********************************************************************//**
 Check that every path from the root to the leaves has the same count and
 the tree nodes are in order.
-@return	TRUE if OK FALSE otherwise */
-UNIV_INTERN
+@return TRUE if OK FALSE otherwise */
 ibool
 rbt_validate(
 /*=========*/
@@ -1350,7 +1330,6 @@ rbt_validate(
 
 /**********************************************************************//**
 Iterate over the tree in depth first order. */
-UNIV_INTERN
 void
 rbt_print(
 /*======*/
